﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;
namespace System.Threading.Tasks
{
        public class MapReduce<KEY, VALUE>
        {
            public MapReduce(IEnumerable<Tuple<KEY, VALUE>> _input)
            {
                input = _input;
            }

            private IEnumerable<Tuple<KEY, VALUE>> input;

            public void Map<KEY2, VALUE2>(Func<Tuple<KEY, VALUE>, IEnumerable<Tuple<KEY2, VALUE2>>> UserDefinedMap,
                out IEnumerable<Tuple<KEY2, VALUE2>> TupleCollection)
            {
                List<Tuple<KEY2, VALUE2>> TupleCollection2 = new List<Tuple<KEY2, VALUE2>>();

                Parallel.ForEach(input, (item) =>
                {
                    Monitor.Enter(this);
                    IEnumerable<Tuple<KEY2, VALUE2>> test = UserDefinedMap(item);
                    TupleCollection2.Capacity = TupleCollection2.Count + test.Count();
                    TupleCollection2.AddRange(test);
                    Monitor.Exit(this);
                });
                TupleCollection = TupleCollection2;
            }

            public void Map<KEY2, VALUE2>(Func<Tuple<KEY, VALUE>, IEnumerable<Tuple<KEY2, VALUE2>>> UserDefinedMap,
                Func<KEY2, VALUE2, bool> Filter, out IEnumerable<Tuple<KEY2, VALUE2>> TupleCollection)
            {

                List<Tuple<KEY2, VALUE2>> TupleCollection2 = new List<Tuple<KEY2, VALUE2>>();

                Parallel.ForEach(input, (item) =>
                {

                    IEnumerable<Tuple<KEY2, VALUE2>> test2 = UserDefinedMap(item);
                    List<Tuple<KEY2, VALUE2>> mylist = test2.ToList();

                    for (int count = 0; count < mylist.Count; ++count)
                    {
                        if (!Filter(mylist[count].Item1, mylist[count].Item2))
                        {
                            mylist.RemoveAt(count);
                        }
                    };

                    IEnumerable<Tuple<KEY2, VALUE2>> test = mylist;
                    Monitor.Enter(this);
                    TupleCollection2.Capacity = TupleCollection2.Count + test.Count();
                    TupleCollection2.AddRange(test);
                   
                    Monitor.Exit(this);
                });

                TupleCollection = TupleCollection2;

            }

            public IEnumerable<Tuple<KEY2, VALUE2>> Reduce<KEY2, VALUE2>(IEnumerable<Tuple<KEY2, VALUE2>> input,
                Func<KEY2, VALUE2[], VALUE2> mapfunction,
                Func<IEnumerable<Tuple<KEY2, VALUE2>>, Dictionary<KEY2, VALUE2[]>> group = null)
            {

                Dictionary<KEY2, VALUE2[]> mapped = new Dictionary<KEY2, VALUE2[]>();

                if (group == null)
                {
                    foreach(Tuple<KEY2, VALUE2>item in input)
                    {
                        if (mapped.ContainsKey(item.Item1))
                        {
                            List<VALUE2> l = new List<VALUE2>(mapped[item.Item1]);
                            l.Add(item.Item2);

                            mapped[item.Item1] = l.ToArray();
                        }
                        else
                        {
                            mapped.Add(item.Item1, new VALUE2[] { item.Item2 });
                        }
                    }
                }
                else
                {
                    mapped = group(input);
                }

                List<Tuple<KEY2, VALUE2>> temp = new List<Tuple<KEY2, VALUE2>>();

                Parallel.ForEach(mapped, (item) =>
                {
                    VALUE2 result = mapfunction(item.Key, item.Value);
                    Monitor.Enter(this);
                    temp.Add(new Tuple<KEY2, VALUE2>(item.Key, result));
                    Monitor.Exit(this);

                });
                return temp;
            }
        }

}
