﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace MapReduce
{
    static class PLINQ
    {
        public static ParallelQuery<TResult> MapReduce<TSource, TMapped, TKey, TResult>(
            this ParallelQuery<TSource> source,
            Func<TSource, IEnumerable<TMapped>> map,
            Func<TMapped, TKey> keySelector,
            Func<IGrouping<TKey, TMapped>, IEnumerable<TResult>> reduce)
        {
            return source.SelectMany(map)
            .GroupBy(keySelector)
            .SelectMany(reduce);
        }
    }

    class Program
    {   
        static void Main(string[] args)
        {
            char [] delimiters={' ', ',', ';', '.'};

           string [] files={    @"C:\shakespeare\Sonett 1.txt",
                                @"C:\shakespeare\Sonett 2.txt",
                                @"C:\shakespeare\Sonett 3.txt", 
                                @"C:\shakespeare\Sonett 4.txt"};

           var counts = files.AsParallel().MapReduce(
                     path => File.ReadLines(path).SelectMany(line => line.Split(delimiters)),
                     word => word,
                     group => new[] { new KeyValuePair<string, int>(group.Key, group.Count()) });

          foreach (var word in counts)
            {
                Console.WriteLine(word.Key + " " + word.Value);
            }

          //Console.WriteLine("Zum Beenden die Eingabetaste drücken");
          //Console.ReadLine();
        }
    }
}
